/* Rowing watch - programma code */
/* CC BY-NC-SA Jeroen Brinkman */
/* Version v1.3 dd 12-4-2025 */


#define VERSION      1.3   // Official software version
#define MINUTE      6000   // One minute in hundreds of seconds (10 milliseconds)
#define MAX   4294967295   // Unsigned long maximum value
#define LIMIT  429400000   // Centiseconds when the Arduino must be reset.
#define DEAF          20   // Number of Centiseconds after pressing that the program will ignore input
#define LONG         200   // Number of Centiseconds for a long press
#define RANGE          9   // Number of splits
#define LOWTEMPO      12   // Minimum tempo
#define HIGHTEMPO     50   // Number of splits
#define OVERTIME    1500   // After 180/LOWTEMPO Centiseconds the stroke rate measurement wille be cancelled
#define BATFULL      830   // Analog read value when the battery is full 4.2V 
#define BATLOW       570   // Analog read value when the battery is empty 2.7V
#define STEP         2.6   // Analog read value for 1% (BATFULL-BATLOW)/100

// RCK is de LATCH pin SCK is de CLOCK pin en DIO (of DIN) is de DATA pin 
#define HEART   1   // Pin for LED on the Nano

#define ROOD    5   // Pin for red button, the stop watch 
#define GROEN   6   // Pin for green button, the tempo watch
#define GEEL    7   // Pin for the yellow button, the split time
#define DRES    8   // Arduino Display Reset-Pin
#define DRS     9   // Arduino Display RS-Pin
#define DCS    10   // Arduino Display CS-Pin  
#define CHARGE A1   // The battery is charging
#define FULL   A2   // The battery is full 
#define BAT    A3   // The battery voltage

#include <SPI.h>             // SPI bus library
#include <Adafruit_GFX.h>    // Adafruit Grafik library
#include <Adafruit_ST7735.h> // Adafruit ST7735 library

Adafruit_ST7735 tft = Adafruit_ST7735(DCS, DRS, DRES);  // Display-library Setup

void(* resetFunc) (void) = 0;

bool timerun = false;    // does the time watch run
bool temporun = false;   // does the tempo watch (base 3) run
bool setsleep = false;   // does the tempo watch (base 3) run
bool r_press, g_press, y_press, split; // are the buttons pressed

unsigned long hs[RANGE];                  // store per Centiseconds when the button is pressed
unsigned long now, temp, tstart;          // Centiseconds
unsigned long t_min, t_sec, t_ten, t_hun; // minutes, seconds, tenth and hundreds of seconds
unsigned long r_time, g_time, y_time;     // last press Centiseconds for buttons
int tempi[RANGE];                         // Temporary storage for measured tempi

long calc;

int s_pointer, t_pointer, i, tempo, bat, showbat;

void setup() {
        
  pinMode(GROEN, INPUT_PULLUP);
  pinMode(ROOD, INPUT_PULLUP);
  pinMode(GEEL, INPUT_PULLUP);
  pinMode(CHARGE, INPUT); 
  pinMode(FULL, INPUT);
  pinMode(HEART, OUTPUT); // blinking led showing the programma's hartbeat

  // initialize variables
  r_time = 0; 
  g_time = 0; 
  y_time = 0;
  now = 0;
  temp = 0;
  tstart = 0;
  t_pointer = 0; // set the tempi pointer to zero
  s_pointer = 0; // set the split time poiter to 0 when the watch starts running
  showbat = (analogRead(BAT)-BATLOW)/STEP;
  for (i = 0; i < RANGE; i++) tempi[i] = 0;

  // initialize the display
  tft.initR(INITR_BLACKTAB);  
  tft.fillScreen(ST7735_BLACK); // make the display black
  tft.setTextSize(2);
  tft.setTextColor(ST7735_WHITE, ST7735_BLACK);
  tft.setCursor(5,5);
  tft.print(" 0:00.0");     
  tft.setTextColor(ST7735_YELLOW, ST7735_BLACK);
  tft.print("  -");
  
  tft.setTextSize(1);
  tft.setTextColor(ST7735_RED, ST7735_BLACK);
  tft.setCursor(0,45);
  tft.print("(C) 2025 J. Brinkman");
  tft.setCursor(0,60);
  tft.print("  Rowing Watch v"); tft.print(VERSION); 
  tft.setCursor(0,90);
  tft.setTextColor(ST7735_YELLOW, ST7735_BLACK);
  tft.print("    Stroke base 3");

}

void loop() {
  digitalWrite(HEART, (millis() / 1000) % 2); //1s heartbeat for the onboard led
  if ((!timerun) && (!temporun) && (millis() > MAX)) resetFunc(); // reset the rowing watch in order to avoid millis() rollover problems occuring once every 50 days

  // Process the buttons, detect if the button is connected to ground therefore invert the value
  g_press = !(digitalRead(GROEN));
  r_press = !(digitalRead(ROOD));
  y_press = !(digitalRead(GEEL));

  now = millis()/10; // the time in hundreds of seconds

  // red button
  if (r_press && (r_time == 0)) {
    r_time = now; // If a press is detected, safe the millis() it happened
    split = true; // set the flag to store the split time
    s_pointer += 1;
    if (!timerun) { // when the rowing watch is not running
      s_pointer = 0; // set the split time pointer to 0 when the watch starts running
      t_pointer = 0; // set the tempi s_pointer to zero
      hs[0] = now; // store the start time per 10ms (one hundred of a second)
      tft.fillScreen(ST7735_BLACK); // make the display black
      tft.setTextSize(2);
      tft.setTextColor(ST7735_YELLOW, ST7735_BLACK);
      tft.setCursor(89,5);
      tft.print("  -");
    }
    timerun = !timerun; // toggle the running of the watch
  }
  if ((now > (r_time + DEAF)) && (!r_press)) r_time = 0;  // if the contact bounce time hass passed and the button is not longer pressed, reset

  // green button
  if (g_press && (g_time == 0) && (s_pointer < 8)) {
    g_time = now; // If a press is detected, safe the Centisecond it happened
    split = true; // set the flag to store the split time
    s_pointer += 1;
    if (timerun) { // when the rowing watch is running
      hs[s_pointer] = now; // store the start time per 10ms (one hundred of a second)
    } else {
      s_pointer = 0; // set the split time poiter to 0 when the watch starts running
      t_pointer = 0; // set the tempi s_pointer to zero
      tempi[0] = 0; // remove tempo clocked without stopwatch running
      hs[0] = now; // store the start time per 10ms (one hundred of a second)
      timerun = true; // Let the rowingwatch run
      tft.fillScreen(ST7735_BLACK); // make the display black
      tft.setTextSize(2);
      tft.setTextColor(ST7735_YELLOW, ST7735_BLACK);
      tft.setCursor(89,5);
      tft.print("  -");
    }
  }
  if ((now > (g_time + DEAF)) && (!g_press)) g_time = 0;  // if the contact bounce hass passed and the button is not longer pressed, reset
  if (s_pointer == 0) split = false; // when the rowingwatch has just been started, no split needs to be stored
  if (s_pointer == 9) split = false; // when maximum storage capacity is reached, ignore the split
  
  // yellow button
  if (y_press && (y_time == 0)) {
    y_time = now; // If a press is detected, safe the millis() it happened
    if (temporun) { // when the tempowatch is running
      temp = round(180 / ((y_time - tstart)/100)); // calculate and store the tempo
      tempi[t_pointer] = temp;
      tft.setTextSize(2); // write the display
      tft.setTextColor(ST7735_YELLOW, ST7735_BLACK);
      tft.setCursor(89,5);
      if ((tempi[t_pointer] >= LOWTEMPO) && (tempi[t_pointer] <= HIGHTEMPO)) { // a valid tempo
        tft.print(" ");
        tft.print(tempi[t_pointer]);
        if (timerun) t_pointer += 1; // increase the tempo pointer when te clock is running
      } else { // a non-valid tempo
        tft.setTextSize(2);
        tft.setTextColor(ST7735_YELLOW, ST7735_BLACK);
        tft.setCursor(89,5);
        tft.print("  -");
      }
      tstart = 0; // set the start time to zero
    } else {
      tstart = now; // set the start time to now
      tft.setTextSize(2);
      tft.setTextColor(ST7735_YELLOW, ST7735_BLACK);
      tft.setCursor(89,5);
      tft.print("  ?");
    }
    temporun = !temporun;
  }
  if ((now > (y_time + 100)) && (!y_press)) y_time = 0;  // if a second has passed and the button is not longer pressed, reset
 
 
  if ((now > (tstart + OVERTIME)) && (temporun)) { // If the yellow button has not been been pressed twice within 15sec, reset the tempo watch
    tstart = 0; temporun = false;
    tft.setTextSize(2);
    tft.setTextColor(ST7735_YELLOW, ST7735_BLACK);
    tft.setCursor(89,5);
    tft.print("  x");
  }

// show the time if the rowing watch runs
  if (timerun) { 
    // Calculate the time in 1/100
    temp = now - hs[0];
    t_min = temp/MINUTE; temp -= (t_min * MINUTE); 
    t_sec = temp/100; temp -= (t_sec * 100); 
    t_ten = temp/10;
    
    // Write the time
    tft.setTextSize(2);
    tft.setTextColor(ST7735_WHITE, ST7735_BLACK);
    tft.setCursor(5,5);
    if (t_min < 10) tft.print(" "); tft.print(t_min); tft.print(":");
    if (t_sec < 10) tft.print("0"); tft.print(t_sec); tft.print(".");
    tft.print(t_ten);
  }
    
  // when it needs to be done show the split time.
  if (split) { 
   
    split = false;
    
    // Calculate the time
    temp = now - hs[s_pointer-1];
    t_min = temp/MINUTE; 
    temp -= (t_min * MINUTE); 

    t_sec = temp/100; 
    temp -= (t_sec * 100); 

    t_hun = temp; 
    
    tft.setTextSize(1);
    tft.setTextColor(ST7735_CYAN, ST7735_BLACK);
    tft.setCursor(0,15+(15*s_pointer));
    tft.print(s_pointer); tft.print(". "); // write the split number
    
    // Write the split time
    tft.setTextColor(ST7735_WHITE, ST7735_BLACK);
    if (t_min < 10) tft.print(" "); tft.print(t_min); tft.print(":");
    if (t_sec < 10) tft.print("0"); tft.print(t_sec); tft.print(".");
    if (t_hun < 10) tft.print("0"); tft.print(t_hun); 
  
    // write the split difference      
    if  (s_pointer == 1) {
      tft.print("        ");
    } else { 
      calc = (now - hs[s_pointer-1]);
      calc += (hs[s_pointer-2] - hs[s_pointer-1]);
      if (calc < 0) {
        tft.setTextColor(ST7735_GREEN, ST7735_BLACK);    
        tft.print(" -");  
        temp = -calc;
      } else {
        if (calc == 0) {
          tft.setTextColor(ST7735_WHITE, ST7735_BLACK);    
          tft.print("  ");
          temp = 0;
        } else { // (calc > 0)
          tft.setTextColor(ST7735_RED, ST7735_BLACK);    
          tft.print(" +"); 
          temp = calc;
        }
      }
      t_sec = temp/100; temp -= (t_sec * 100); 
      t_hun = temp; 

      if (t_sec > 99) {
        tft.print(t_sec); tft.print("  ");
      } else {
        if (t_sec < 10) tft.print(" "); tft.print(t_sec); tft.print(".");
        if (t_hun < 10) tft.print("0"); tft.print(t_hun); tft.print(" ");         
      }
    }
    
    // write the average stroke rate 
    tft.setTextColor(ST7735_YELLOW, ST7735_BLACK);
    if (tempi[0] == 0) {
      tft.print(" -");
    } else {
      i = 0; tempo = 0;
      for (i = 0; i < t_pointer; i++) {
        if(tempi[i] != 0) {
          tempo += tempi[i];
          tempi[i] = 0;
        }
      }
      tempo = round(tempo / i);
      if (tempo < 10) tft.print(" ");  
      if ((tempo >= LOWTEMPO) && (tempo <= HIGHTEMPO)) tft.print(tempo);  // print a valid tempo
      else tft.print("-");
    }   
    t_pointer = 0; // reset the tempo split pointer
  }
  
  // show the battery status 
  tft.setCursor(0,150);
  tft.setTextSize(1);
  tft.setTextColor(ST7735_CYAN, ST7735_BLACK);
  tft.print("Battery: ");
  bat = (analogRead(BAT)-BATLOW)/STEP;
  if (abs(bat-showbat) > 1) showbat = bat;
  if (showbat > 0) {
    tft.print(showbat); 
    tft.print("% ");  
    if (digitalRead(FULL) == 1) tft.print("Full "); 
    else if (digitalRead(CHARGE) == 1) tft.print("Charge");    
      else tft.print("        ");  
  } else   tft.print("Off        ");
} // loop
